@@ -148,18 +148,27 @@ def run_program(window):
     print 'Command to execute: \n' + ' '.join(str(x) for x in cmdline)
     return cmdline
 
+
 # Thread for RDP client launch feedback in UI
 class WorkerThread(threading.Thread):
     def __init__(self, callback, cmdline):
         threading.Thread.__init__(self)
         self.callback = callback
         self.cmdline = cmdline
+        WorkerThread.error_text = ''
+        WorkerThread.return_code = 0
 
     # Start the client and wait some seconds for errors
     def run(self):
-        global p
         p = subprocess.Popen(self.cmdline, stderr=subprocess.PIPE)
-        time.sleep(2)
+        start_time = time.time()
+        while p.poll() is None:
+            time.sleep(1)
+            if time.time() - start_time > 3:
+                break
+        if p.poll() is not None:
+            WorkerThread.error_text += p.communicate()[1]
+            WorkerThread.return_code += p.returncode
         GLib.idle_add(self.callback)
 
 
@@ -240,7 +249,7 @@ e.g. "1024x768" or "80%"''')
         self.clioptionsentry = Gtk.Entry()
         self.clioptionsentry.set_tooltip_text('''Extra CLI options''')
         self.clioptionsentry.connect("activate",
-                                   self.enter_connect, self.clioptionsentry)
+                                     self.enter_connect, self.clioptionsentry)
 
         # Combobox for program selection
         program_store = Gtk.ListStore(str)
@@ -410,9 +419,14 @@ e.g. "1024x768" or "80%"''')
         self.spinner.stop()
         self.spinner.hide()
         self.connectbutton.show()
-        if p.poll() is not None:
+        error_text = WorkerThread.error_text
+        return_code = WorkerThread.return_code
+        # Display error popup in UI if 300 characaters or less
+        if return_code is not 0:
+            if len(error_text) > 300:
+                error_text = error_text[:300] + '...'
             self.on_warn(None, 'Connection Error', '%s: \n' % client +
-                         p.communicate()[1])
+                         error_text)
 
     # Triggered when the combobox is clicked.  We load the selected profile
     # from the config file.
@@ -443,7 +457,8 @@ e.g. "1024x768" or "80%"''')
 
     # Triggered when the file menu is used
     def add_file_menu_actions(self, action_group):
-        action_filemenu = Gtk.Action(name="FileMenu", label="File", tooltip=None, stock_id=None)
+        action_filemenu = Gtk.Action(name="FileMenu", label="File",
+                                     tooltip=None, stock_id=None)
         action_group.add_action(action_filemenu)
         # Why do the functions here execute on startup if we add parameters?
         action_group.add_actions([("SaveCurrentConfig", None,
@@ -455,7 +470,8 @@ e.g. "1024x768" or "80%"''')
         action_group.add_actions([("DeleteCurrentConfig", None,
                                    "Delete Current Profile", None, None,
                                    self.delete_current_config)])
-        action_filequit = Gtk.Action(name="FileQuit", label=None, tooltip=None, stock_id=Gtk.STOCK_QUIT)
+        action_filequit = Gtk.Action(name="FileQuit", label=None, tooltip=None,
+                                     stock_id=Gtk.STOCK_QUIT)
         action_filequit.connect("activate", self.quit)
         action_group.add_action(action_filequit)
 
@@ -533,7 +549,8 @@ e.g. "1024x768" or "80%"''')
 
     # Generic warning dialog
     def on_warn(self, widget, title, message):
-        dialog = Gtk.MessageDialog(transient_for=self, flags=0, message_type=Gtk.MessageType.WARNING,
+        dialog = Gtk.MessageDialog(transient_for=self, flags=0,
+                                   message_type=Gtk.MessageType.WARNING,
                                    buttons=Gtk.ButtonsType.OK, text=title,
                                    title='Rocket Depot')
         dialog.format_secondary_text(message)
@@ -544,7 +561,7 @@ e.g. "1024x768" or "80%"''')
     def on_about(self, widget):
         about = Gtk.AboutDialog()
         about.set_program_name("Rocket Depot")
-        about.set_version("0.16")
+        about.set_version("0.17")
         about.set_copyright("2014 David Roble")
         about.set_comments("rdesktop/xfreerdp Frontend")
         about.set_website("https://github.com/robled/rocket-depot")
